//
// GStack.h - implementation of a fast and memory efficient
//            generic stack class which stores pointers to objects.
//            The user must subclass to make a version specific to
//            the type of object they wish to store.  Details can be
//            found in the test program appended to GStack.C.
//
// $Id: GStack.h,v 1.4 1993/06/20 01:51:53 lijewski Exp $
//
// Copyright 1991, 1992, 1993 by Mike Lijewski.
//

#ifndef GENERIC_STACK_H
#define GENERIC_STACK_H


class GStack
{
  public:

    GStack (void);
    GStack (const GStack&);
    ~GStack (void);

    void operator= (const GStack&);

    void push (void* object);
    void* pop (void);

    void* top () const;

    unsigned long entries (void) const;
    
    int isEmpty (void) const;

  private:

    enum { BlockSize = 10, ArraySize = 5 };

    void*** blocks;
    int index;              // index into a block of arrays of void*s
    int offset;             // offset into block for push and pop
    unsigned long size;     // number of allocated block pointers
    unsigned long nentries; // number of entries in GStack
};

inline unsigned long GStack::entries (void) const { return nentries; }

inline int GStack::isEmpty (void) const { return nentries == 0; }

inline void* GStack::top () const
{
    return isEmpty() ? 0 : blocks[index][offset==0 ? BlockSize-1 : offset-1];
}

#endif /*GENERIC_STACK_H*/
