//
// classes.C - contains definitions of the member functions which
//             are not defined in the relevant class declarations.
//
// $Id: classes.C,v 1.6 1993/06/20 01:51:53 lijewski Exp $
//
// Copyright (c) 1991, 1992, 1993 by Mike Lijewski.
//

#include <new.h>
#include <stdio.h>
#include <string.h>

#include "classes.h"
#include "dired.h"


DirList::DirList (char *name)
{
    _head = _tail = 0;
    _next = _prev = 0;
    _nelems = 0;
    _name = name;
}


//
// Inserts a new DirLine, constructed from the given char**, after
// the current line in the DirList.
//

DirLine *DirList::insert (char **line)
{
    DirLine *new_line = new DirLine(line);
    DirLine *ln = currLine();

    if (atEndOfList())
    {
        ln->_next = new_line;
        new_line->_prev = ln;
        _tail = new_line;
    }
    else
    {
        new_line->_next = ln->next();
        new_line->_prev = ln;
        ln->_next->_prev = new_line;
        ln->_next = new_line;
    }
    _nelems++;
    return new_line;
}


//
// Adds the DirLine to the listing maintained by DirList.
//

void DirList::add (DirLine *link)
{
    if (nelems())
    {
        _tail->_next = link;
        _tail->_next->_prev = tail();
        _tail = link;
        _nelems++;
    }
    else
    {
        _head = _tail = link;
        _nelems = 1;
    }
}


//
// Delete the current listing line in the window and update our view.
// The width of our view always decreases by one.  If the calling procedure
// adds more lines to the screen, they will have to reset
// lastLine and/or firstLine, but currLine does not need to change.
//

void DirList::deleteLine ()
{
    DirLine *line = currLine();

    if (atBegOfList())
    {
        //
        // that is, firstLine() == head()
        //
        _head = _firstLine = _currLine = head()->next();
        _head->_prev = 0;
    }
    else if (atWindowTop())
    {
        //
        // but firstLine() != head()
        //
        _firstLine = _currLine = line->next();
        line->_next->_prev = line->prev();
        line->_prev->_next = line->next();
    }
    else if (atEndOfList())
    {
        //
        // lastLine() == tail()
        //
        _tail = _lastLine = _currLine = line->prev();
        _tail->_next = 0;
    }
    else
    {
        _currLine = line->next();
        line->_next->_prev = line->prev();
        line->_prev->_next = line->next();
    }

    _nelems--;
    delete line;
}


DirList::~DirList ()
{
    if (nelems())
    {
        DirLine *tmp = tail(), *prev = tail()->prev();
        while(tmp)
        {
            delete tmp;
            if ((tmp = prev) != 0)
                prev = tmp->prev();
        }
        delete tmp;
        delete [] _name;
    }
}


//
// The definition of the head of the freelist that DirLine::operator new()
// uses to dole out dirLines efficiently.
//

DirLine *DirLine::freeList;

typedef void (*PEHF)();

void *DirLine::operator new (size_t size)
{
    if (size != sizeof(DirLine))
        return ::new char[size];

    DirLine *line = freeList;

    if (line)
        freeList = line->next();
    else
    {
        //
        // We rely on a freestore exception catcher to catch
        // heap allocation failures.
        //
        DirLine *block = (DirLine *) ::new char[chunksize * sizeof(DirLine)];

        for (int i = 0; i < chunksize - 1; i++)
            block[i]._next = (DirLine *)&block[i + 1];
        block[chunksize - 1]._next = 0;
        line = block;
        freeList = &block[1];
    }
    return line;
}


void DirLine::operator delete (void *object)
{
    DirLine *line = (DirLine *)object;
    line->_next = freeList;
    freeList = line;
}
