//
// dired.h - global declarations
//
// $Id: dired.h,v 1.9 1993/09/19 20:28:52 lijewski Exp $
//

#ifndef DIRED_H
#define DIRED_H


#include <stdio.h>
#include <string.h>

#include "String.h"
#include "keys.h"
#include "classes.h"
#include "commands.h"


#ifdef apollo
extern "C" int putenv(const char*);
#endif
#if defined(linux) || defined(__sgi)
#define major(x) (int)(((unsigned)(x)>>8)&0x7F)
#define minor(x) (int)((x)&0xFF)
#endif
#ifdef __sgi
extern "C" int ioctl( int fildes, int request, ...);
#endif
#if defined(apollo) || defined(__sgi)
#define SIGFUNC(f) void f(int ...)
#else
#define SIGFUNC(f) void f(int)
#endif


//
// The four ways we sort a directory listing.
//
enum sort_order { ALPHABETICALLY,
                  MODIFICATION_TIME,
                  ACCESS_TIME,
                  INODE_CHANGE_TIME };

//
// GLOBALS
//
extern DirStack *dir_stack;                 // our directory stack
extern sort_order how_to_sort;              // default sort order
extern const char *const ls_cmd[4];         // command to get long listing
extern const char *const modeline_prefix;   // modeline prefix
extern const char *const version;           // our version number
extern int win_size_changed;                // window size has changed?
extern int message_window_dirty;            // something in message window?


//
// For POSIX compatibility.
//
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif


//
// from `commands.C'
//
extern void read_commands (DirList *, const KeyMap&);


//
// from `dired.C'
//
extern void dired (const char *dirname, const KeyMap&);


//
// from `utilities.C'
//
extern char       *current_directory (void);
extern const char* decode (const char* str);
extern void        display_string (const char *str, size_t len = 0); 
extern void        eat_a_character (const char *msg);
extern const char* encode (char* str);
extern void        error (const char *str);
extern void        error (const char *fmt, int index);
extern void        error (const char *fmt, const char *file, int line);
extern void        error (const char *fmt, const char *file, int line,
                          const char *str);
extern void        exec_with_system (const char *cmd, int prompt = 1);
extern int         execute (const char *file, const char *argv[], int closem=1);
extern const char *expand_tilde (const char *str);
extern char       *fgetline (FILE *fp);
extern DirList    *get_directory_listing (char *dirname);
extern const char *get_file_name (DirList *list);
extern int         goal_column (DirList *l);
extern void        init_screen_and_kbdr (void);
extern void        initialize (void);
extern void        initial_listing (DirList *dl);
extern int         is_directory (const char *dir);
extern int         is_regular_file (const char *file);
extern void        leftshift_current_line (DirList *dl);
extern int         lines_displayed (DirList *dl);
extern void        message (const char *fmt, const char *str = 0);
#ifdef COMPLETION
extern const char *prompt (const char *msg, int do_completion = 0);
#else
extern const char *prompt (const char *msg);
#endif
extern             SIGFUNC(exit_nicely);
extern int         read_and_exec_perm (const char *dir);
extern char        read_from_keybd (void);
extern void        redisplay (void);
extern void        rightshift_current_line (DirList *dl);
extern void        set_signals (void);
extern char       *BMstrstr (const char *, const char *);
extern int         tokenize (const char *line,
                             const char *separators,
                             char**& tokens);
extern void        update_modeline (const char * = 0, const char * = 0);
extern void        unset_signals (void);
extern             SIGFUNC(winch);


// max - the maximum of two integer arguments.
inline int max(int x, int y) { return x >= y ? x : y; }


// set_sort_order - sets the sorting order.
inline void set_sort_order(sort_order order) { how_to_sort = order; }


// the_sort_order - returns the sorting order
inline sort_order the_sort_order(void) { return how_to_sort; }

#endif /*DIRED_H*/
