//
// keys.h - contains declarations needed for virtual keyboard.
//
// $Id: keys.h,v 1.8 1993/06/20 01:51:53 lijewski Exp $
//
// Copyright (c) 1991, 1992, 1993 by Mike Lijewski.
//

#ifndef KEYS_H
#define KEYS_H


//
// Valid keyboard generated commands.
//
enum CommandCode
{
    CURSOR_DOWN,
    CURSOR_UP,
    PAGE_UP,
    PAGE_DOWN,
    HALFPAGE_UP,
    HALFPAGE_DOWN,
    TOP_OF_FILE,
    BOTTOM_OF_FILE,
    EDIT_FILE,
    PAGE_FILE,
    COPY_FILE,
    DELETE_FILE,
    RENAME_FILE,
    COMPRESS_FILE,
    EDIT_DIRECTORY,
    CHGRP_FILE,
    HELP,
    LINK_FILE,
    CHMOD_FILE,
    CHANGE_SORT_ORDER,
    PRINT_FILE,
    REREAD_DIRECTORY,
#ifndef NO_SYMLINKS
    SYMLINK_FILE,
#endif
    GUNZIP_FILE,
    UNCOMPRESS_FILE,
    SEARCH_FORWARD,
    SEARCH_BACKWARD,
    SHELL_COMMAND,
    VERSION,
    GZIP_FILE,
    REDISPLAY,
    POP_THIS_DIRECTORY,
    QUIT,
    UNKNOWN 
};


//
// This is a forward definition an array of help strings for
// each of the functions.  This assumes that the CommandCode
// enums are integers from 0 through UNKNOWN.
//

extern const char* Help[];


//
// Ordered pair of a CommandCode with the name of the human-readable
// name of the command.  Used only when initializing a keymap.
//

struct Command
{
    CommandCode keycode;
    char* name;
};


//
// This maps a CommandCode to the sequence of characters which must
// be typed at the keyboard to invoke the command.  Also holds the help
// message for the command.
//

struct EncodedCommand
{
    CommandCode keycode;
    String sequence;
    const char* help_msg;

    EncodedCommand (CommandCode c, const char* s)
        :keycode(c), sequence(s), help_msg(Help[c]) { }
    EncodedCommand (CommandCode c, const String& s)
        :keycode(c), sequence(s), help_msg(Help[c]) { }
};


//
// Class modelling a KeyMap.
//

class KeyMap
{
  public:

    KeyMap ();
    ~KeyMap ();

    KeyMap (const KeyMap& rhs);

    KeyMap& operator= (const KeyMap& rhs);

    void add (CommandCode code, const char*   seq);
    void add (CommandCode code, const String& seq);

    int entries () const { return nentries; }
    const EncodedCommand* operator[] (int i) const { return map[i]; }

    const EncodedCommand** Map() const { return map; }

  private:

    int nentries;
    int size;
    EncodedCommand** map;
};


//
// Exported functions.
//

extern void        initialize_keymap_from_file (KeyMap& k, const char* file);
extern CommandCode get_command (const KeyMap& keymap);

#endif /*KEYS_H*/
